
/* Generic XML Function To Grab The Text From An Element. */

FUNCTION get-text RETURNS CHARACTER (INPUT NodeHandle AS HANDLE).
    
    DEFINE VARIABLE TextHandle AS HANDLE NO-UNDO.
    DEFINE VARIABLE k AS INTEGER.
    
    CREATE X-NODEREF TextHandle.
    
    /* This Node may have other children, remember to
       only look for its text. */
    
    DO k = 1 TO NodeHandle:NUM-CHILDREN:
       NodeHandle:get-child(TextHandle,k).
       IF TextHandle:NAME = "#TEXT" THEN DO:
          RETURN TextHandle:NODE-VALUE. 
       END.
    END. /* each child */
    
    DELETE OBJECT TextHandle.
    
END FUNCTION. /* get-text function */

/* Generic XML Function To Save A Value to the Text Of An Element. */

FUNCTION set-text RETURNS LOGICAL (INPUT NodeHandle AS HANDLE,
                                   INPUT TextString AS CHARACTER).
                   
DEFINE VARIABLE TextHandle AS HANDLE NO-UNDO.
DEFINE VARIABLE k AS INTEGER.
    
    CREATE X-NODEREF TextHandle.
    /* This Node may have other children, remember to
       only look for its text. */
    DO k = 1 TO NodeHandle:NUM-CHILDREN:
       NodeHandle:get-child(TextHandle,k).
       IF TextHandle:NAME = "#TEXT" THEN DO:
          TextHandle:NODE-VALUE = TextString.
          DELETE OBJECT TextHandle.    
          RETURN YES.
       END. /* text node */
    END. /* each child */
    
    /* If the program gets this far, the Text node was not found.
       Also means the Object wasn't deleted either */
    
    DELETE OBJECT TextHandle.    
    RETURN NO.

END FUNCTION. /* get-text function */

/* Generic XML Function to add a new node to the over all
   document, and set its value. */

FUNCTION new-child-node RETURNS HANDLE (INPUT DocumentHandle AS HANDLE,
                                        INPUT ParentHandle AS HANDLE,
                                        INPUT NodeName AS CHARACTER,
                                        INPUT NodeValue AS CHARACTER).

DEFINE VARIABLE NewNodeHandle AS HANDLE NO-UNDO.
DEFINE VARIABLE NewTextHandle AS HANDLE NO-UNDO.
    
    CREATE X-NODEREF NewNodeHandle.
    CREATE X-NODEREF NewTextHandle.
    
    /* Create a new node, this is always against the whole
       document, not against the parent.  Then link to
       the parent in a second step. */
       
    DocumentHandle:CREATE-NODE(NewNodeHandle,NodeName,"ELEMENT").
    ParentHandle:APPEND-CHILD(NewNodeHandle).
    
    /* This is now the text node that has to be created separately.
       Notice how the link is done underneath the new node,
       not under neath the parent node. */
          
    DocumentHandle:CREATE-NODE(newTextHandle,"","TEXT").
    NewTextHandle:NODE-VALUE = NodeValue.
    NewNodeHandle:APPEND-CHILD(NewTextHandle).
    
    DELETE OBJECT NewNodeHandle.
    DELETE OBJECT NewTextHandle.
    RETURN NewNodeHandle.
    
END FUNCTION.

/* Quick and Easy way to switch between Screen and Web
   versions of the node traversal. Simply uncomment
   the other version -- */             
             
PROCEDURE ShowDocument.
DEFINE INPUT PARAMETER DocumentName AS CHARACTER NO-UNDO.

   RUN ScreenShowDocument(INPUT DocumentName).
   /* RUN WebShowDocument(INPUT DocumentName). */

END. /* Procedure ShowDocument */

PROCEDURE ScreenShowDocument.

DEFINE INPUT PARAMETER DocumentName AS CHARACTER NO-UNDO.
DEFINE VARIABLE XDocHandle AS HANDLE NO-UNDO.
DEFINE VARIABLE XNodeHandle AS HANDLE NO-UNDO.

         
CREATE X-DOCUMENT XDocHandle.

XdocHandle:LOAD("file",DocumentName,FALSE).

CREATE X-NODEREF XNodeHandle.

XDocHandle:GET-DOCUMENT-ELEMENT(XNodeHandle).

DISPLAY "Root Name:" xnodehandle:NAME SKIP
                  "Children:" xnodehandle:NUM-CHILDREN SKIP
                  "Value:" xnodehandle:NODE-VALUE SKIP
                  "Type:" xnodehandle:TYPE SKIP.
        
RUN ScreenShowNode (INPUT XNodeHandle).        
        
DELETE OBJECT xnodehandle.
DELETE OBJECT xdochandle.

END.  /* Procedure ScreenShowDocument */

PROCEDURE ScreenShowNode.

DEFINE INPUT PARAMETER ParentHandle AS HANDLE no-undo.
DEFINE VARIABLE ChildHandle AS HANDLE NO-UNDO.
DEF VAR i AS INTEGER NO-UNDO.

CREATE X-NODEREF ChildHandle.

DO i = 1 TO ParentHandle:NUM-CHILDREN:

   ParentHandle:GET-CHILD(ChildHandle,i).
   DISPLAY SKIP(1) "Child Name:" ChildHandle:NAME SKIP
        "Children:" ChildHandle:NUM-CHILDREN SKIP
        "Value:" ChildHandle:NODE-VALUE SKIP
        "Type:" ChildHandle:TYPE SKIP.
   
RUN ScreenShowNode (INPUT ChildHandle).

END. /* I = 1 to number of children */

DELETE OBJECT childhandle.     
   
END. /* Procedure ScreenShowNode */

PROCEDURE WebShowDocument.

DEFINE INPUT PARAMETER DocumentName AS CHARACTER NO-UNDO.
DEFINE VARIABLE XDocHandle AS HANDLE NO-UNDO.
DEFINE VARIABLE XNodeHandle AS HANDLE NO-UNDO.

         
CREATE X-DOCUMENT XDocHandle.

XdocHandle:LOAD("file",DocumentName,FALSE).

CREATE X-NODEREF XNodeHandle.

XDocHandle:GET-DOCUMENT-ELEMENT(XNodeHandle).

{&OUT} "Root Name:" xnodehandle:NAME "<BR>"
                  "Children:" xnodehandle:NUM-CHILDREN "<BR>"
                  "Value:" xnodehandle:NODE-VALUE "<BR>"
                  "Type:" xnodehandle:TYPE "<BR>".
        
RUN WebShowNode (INPUT XNodeHandle).        
        
DELETE OBJECT xnodehandle.
DELETE OBJECT xdochandle.

END.  /* Procedure WebShowDocument */

PROCEDURE WebShowNode.

DEFINE INPUT PARAMETER ParentHandle AS HANDLE no-undo.
DEFINE VARIABLE ChildHandle AS HANDLE NO-UNDO.
DEF VAR i AS INTEGER NO-UNDO.

CREATE X-NODEREF ChildHandle.

DO i = 1 TO ParentHandle:NUM-CHILDREN:

   ParentHandle:GET-CHILD(ChildHandle,i).
   {&OUT} "<HR>" "Child Name:" ChildHandle:NAME "<BR>"
        "Children:" ChildHandle:NUM-CHILDREN "<BR>"
        "Value:" ChildHandle:NODE-VALUE "<BR>"
        "Type:" ChildHandle:TYPE "<BR>".
   
RUN WebShowNode (INPUT ChildHandle).

END. /* I = 1 to number of children */

DELETE OBJECT childhandle.     
   
END. /* Procedure WebShowNode */

/* Generic 4GL Utility to translate an XML Doc
   into a dynamic Temp Table */

PROCEDURE XML2TT.

DEFINE INPUT PARAMETER DocumentFileName AS CHARACTER NO-UNDO.
DEFINE OUTPUT PARAMETER TTHandle AS HANDLE NO-UNDO.

DEFINE VARIABLE BufferHandle AS HANDLE NO-UNDO.

DEFINE VARIABLE DocumentHandle AS HANDLE NO-UNDO.
DEFINE VARIABLE RootNodeHandle AS HANDLE NO-UNDO.
DEFINE VARIABLE ChildNodeHandle AS HANDLE NO-UNDO.

DEFINE VARIABLE i AS INTEGER NO-UNDO.

CREATE TEMP-TABLE TTHandle.
TTHandle:TEMP-TABLE-PREPARE("XML").
BufferHandle = TTHandle:DEFAULT-BUFFER-HANDLE.


CREATE X-DOCUMENT DocumentHandle.
CREATE X-NODEREF RootNodeHandle.
CREATE X-NODEREF ChildNodeHandle.


DocumentHandle:LOAD("file", DocumentFileName, FALSE).
DocumentHandle:GET-DOCUMENT-ELEMENT(RootNodeHandle).


REPEAT i = 1 TO RootNodeHandle:NUM-CHILDREN:

   RootNodeHandle:GET-CHILD(ChildNodeHandle,i).
   BufferHandle:BUFFER-CREATE().
   
   RUN ProcessNode (INPUT BufferHandle, 
                             INPUT ChildNodeHandle).
   
END.

DELETE OBJECT ChildNodeHandle.
DELETE OBJECT RootNodeHandle.
DELETE OBJECT DocumentHandle.

DELETE OBJECT BufferHandle.                       
DELETE OBJECT TTHandle.

END. /* Procedure XML2TT */

PROCEDURE ProcessNode.

   DEFINE INPUT PARAMETER BufferHandle AS HANDLE NO-UNDO.
   DEFINE INPUT PARAMETER NodeHandle AS HANDLE NO-UNDO.
   
   DEFINE VARIABLE AttributeName AS CHARACTER NO-UNDO.   
   DEFINE VARIABLE ChildNode AS HANDLE NO-UNDO.                                          
   DEFINE VARIABLE i AS INTEGER NO-UNDO.
   
   DO i = 1 TO NUM-ENTRIES(NodeHandle:ATTRIBUTE-NAMES):
      
      AttributeName = ENTRY(i,NodeHandle:ATTRIBUTE-NAMES).
      RUN SaveField (INPUT BufferHandle,
                     INPUT AttributeName,
                     INPUT NodeHandle:GET-ATTRIBUTE(AttributeName)).
      
      
   END. /* Attribute Names */
   
   RUN SaveField (INPUT BufferHandle,
                  INPUT NodeHandle:NAME,
                  INPUT get-text(NodeHandle)).
                  
   DO i = 1 TO NodeHandle:NUM-CHILDREN:
      
      NodeHandle:GET-CHILD(ChildNode,i).
      IF ChildNode:NAME <> "#text" THEN DO:
         RUN ProcessNode(INPUT BufferHandle,
                         INPUT ChildNode).
             
      END.
   END.

END. /* ProcessNode Procedure */

PROCEDURE SaveField.
   
   DEFINE INPUT PARAMETER BufferHandle AS HANDLE NO-UNDO.
   DEFINE INPUT PARAMETER FieldName AS CHARACTER NO-UNDO.
   DEFINE INPUT PARAMETER FieldValue AS CHARACTER NO-UNDO.
   
   DEFINE VARIABLE FieldHandle AS HANDLE NO-UNDO.
                                           
      FieldHandle = BufferHandle:BUFFER-FIELD(FieldName).
      IF FieldHandle = ? THEN DO:
         BufferHandle:ADD-NEW-FIELD(FieldName,"Character").
      END.
      FieldHandle:BUFFER-VALUE = FieldValue.

END. /* SaveField Procedure */




